<?php
//Googleアナリティクスから昨日から過去30日間のPV数・ページパス・ページ名を取得します。
//ページタイトルが「notset」になってしまったものにも対応しています。
//pv数順になるようソートを実装してあります。
//同順位に対応しています。
//Composerでgoogle/apiclient:^2.0を使用しています。
//Composer配布サイト(https://getcomposer.org/)
//Composerがインストールされているか確認するには作業ディレクトリにて[cmd:php composer.phar]で確認出来ます。
//apiのダウンロードは当ファイルと同じディレクトリで[cmd:php composer.phar require google/apiclient:^2.0]を実行して下さい。
//apiのダウンロード後、同じディレクトリにvendorフォルダが出来た事を確認して下さい。
//service-account-credentials.jsonは同じディレクトリに設置して下さい。
//変更箇所はビューID1か所です。
//※cmd=コマンドプロンプト
//**************************************************************************************
//初めての方は色々準備が面倒と思いますがファイトでーす(n*´ω`*n)
//**************************************************************************************
require_once __DIR__ . '/vendor/autoload.php';
$analytics = initializeAnalytics();
$response  = getReport($analytics);
printResults($response);
function initializeAnalytics(){
  $KEY_FILE_LOCATION = "service-account-credentials.json";
  $client = new Google_Client();
  $client->setAuthConfig($KEY_FILE_LOCATION);
  $client->setScopes(['https://www.googleapis.com/auth/analytics.readonly']);
  $analytics = new Google_Service_AnalyticsReporting($client);
  return $analytics;
}
//レポートの取得
function getReport($analytics){
  //ビューIDの指定
  $VIEW_ID = "view_id";
  //ＰＶ数をリクエスト
  $values = new Google_Service_AnalyticsReporting_Metric();
  $values->setExpression("ga:pageviews");
  $values->setAlias("pv");
  //ページタイトル・ページパスをリクエスト
  $dimensions[0] = new Google_Service_AnalyticsReporting_Dimension();
  $dimensions[1] = new Google_Service_AnalyticsReporting_Dimension();
  $dimensions[0]->setName("ga:pageTitle");
  $dimensions[1]->setName("ga:pagePath");
  // データ取得期間の自動挿入設定※昨日から過去30日間
  date_default_timezone_set("Asia/Tokyo");
  $sdate = date("Y-m-d",strtotime("-31 day"));//昨日から30日前の日付
  $edate = date("Y-m-d",strtotime("-1  day"));//昨日の日付
  $dateRange = new Google_Service_AnalyticsReporting_DateRange();
  $dateRange->setStartDate($sdate);
  $dateRange->setEndDate($edate);
  //収集データのリクエスト内容をまとめる
  $resource = new Google_Service_AnalyticsReporting_ReportRequest();
  $resource->setViewId($VIEW_ID);
  $resource->setDateRanges($dateRange);
  $resource->setDimensions([array($dimensions[0]),$dimensions[1]]);
  $resource->setMetrics(array($values));
  $body = new Google_Service_AnalyticsReporting_GetReportsRequest();
  $body->setReportRequests(array($resource));
  return $analytics->reports->batchGet($body);
}
//printResults()のレスポンスは配列（JSON形式）で帰ってくる
//{reports:[columnHeader{...},date:{rows:[...]}]}
//rows:[{{dimension:[ページ名,ページパス],metric:[{values:[pv数]}]}},{...}]
function printResults($response){
  $report = $response[0];
  $header = $report->getColumnHeader();
  $dimensionHeaders = $header->getDimensions();
  $metricHeaders = $header->getMetricHeader()->getMetricHeaderEntries();
  $rows = $report->getData()->getRows();
  //受け取ったデータの分解
  for($i = 0; $i < count($rows); $i++){
    $row = $rows[$i];
    //ページタイトル
    $pn = $row->getDimensions()[0];
    //ページパス
    $pp = $row->getDimensions()[1];
    //PV数
    $pv = $row->getMetrics()[0]->getValues()[0];
    //１データ単位を最終データ配列に格納
    $itemArray =  array("rank" => 0);//順位格納用オブジェクトの追加
    $itemArray += array("pn" => $pn);//$pnをitemArrayに追加
    $itemArray += array("pp" => $pp);//$ppをitemArrayに追加
    $itemArray += array("pv" => $pv);//$pvをitemArrayに追加
    $rankArray[] = $itemArray;
  }
  //同ページパスのオブジェクトをページ名が正規のオブジェクト側に合成
  $check = "/(合成済み重複要素|not set)/";
  for($i = 0; $i < count($rankArray); $i++){
    //pnが"変数check"ならスキップ
    if(preg_match($check,$rankArray[$i]["pn"])){continue;}
    for($j = 0; $j < count($rankArray); $j++){
      //インデックス番号が同じならスキップ
      if($i == $j){continue;}
      //ページパスが同じなら合成
      if($rankArray[$i]["pp"] == $rankArray[$j]["pp"]){
        $rankArray[$i]["pv"] = $rankArray[$i]["pv"] + $rankArray[$j]["pv"];
        $rankArray[$j]["pn"] = "合成済み重複要素";
      }
    }
  }
  //合成済み重複要素とページ名が「notset」で余ったものを除去
  $len = count($rankArray) - 1;
  for ($i = $len; $i >= 0; $i--){
    if(preg_match($check,$rankArray[$i]["pn"])){//pnが"変数check"なら除去
      array_splice($rankArray, $i, 1);
      continue;
    }
  }
  //pv数順にソート
  foreach($rankArray as $key => $row){
    $amsKey[$key] = $row["pv"];
  }
  array_multisort($amsKey, SORT_DESC, $rankArray);
  //順位付けと同順位処理
  $rank  = 0;
  $stock = 0;
  $stpv  = 0;
  for($i = 0; $i < count($rankArray); $i++){
    //同順位処理
    if($rankArray[$i]["pv"] < $stpv || $i == 0){//[上位と同じpv数]または[iが０]なら
      $rank = $rank + 1 + $stock;
      $stock = 0;
    }else{
      $stock = 1;
    }
    $rankArray[$i]["rank"] = $rank;//順位の書き込み
    $stpv = $rankArray[$i]["pv"];//比較用pv数の一時保管
  };
  //出力
  print_r($rankArray);
}